from osv import osv,fields
import time
from mx import DateTime
import datetime
import netsvc
from tools.translate import _

class purchase_request(osv.osv):
    _name="purchase.request"

    _columns={
        "name": fields.char("Doc No",size=64,select=1, readonly=1),
        "date": fields.date("Created",readonly=True,select=1),
        "required_date" : fields.date("Required Date"),
        "origin":fields.char("Origin",size=16,select=2,readonly=1, states={'draft':[('readonly',False)]} ),
        "requester_id": fields.many2one("res.users","Requested By",readonly=True),
        "department_id": fields.many2one("hr.department","Department",readonly=True),
        "location_id": fields.many2one("stock.location","Location",required=True),
        "state": fields.selection([("draft","Draft"),("wait_head","Waiting Head for Approval"),
                                    ("wait_gm","Waiting GM for Approval"),("approved","Approved"),
                                    ("disapproved","Disapproved"),("ordered","Ordered"),
                                    ("received","Received"),('done','Done'),("canceled","Canceled")],"Status",readonly=True,select=1),
        "desc": fields.char("Description",size=64,required=True,select=1 ,readonly=1, states={'draft':[('readonly',False)]} ),
        "product_id": fields.many2one("product.product","Product", readonly=1, states={'draft':[('readonly',False)],'approved':[('readonly',False)]} ),
        "supplier_id": fields.many2one("res.partner","Supplier",select=1),
        "qty": fields.float("Quantity",required=True,readonly=1, states={'draft':[('readonly',False)]} ),
        "uom_id": fields.many2one("product.uom","UoM",required=True ,readonly=1, states={'draft':[('readonly',False)]} ),
        "po_id": fields.many2one("purchase.order","Purchase Order"),
        "reserv_id": fields.many2one("stock.move","Reservation"),
        "notes": fields.text("Notes"),
    }

    _order="id desc"

    def _get_department(self,cr,uid,context):
        cr.execute("select department_id from hr_department_user_rel where user_id=%d"%uid)
        res=cr.fetchone()
        if not res:
            return False
        return res[0]

    def _get_location(self,cr,uid,context):
        res=self.pool.get("stock.warehouse").search(cr,uid,[])
        if not res:
            return False
        wh_id=res[0]
        wh=self.pool.get("stock.warehouse").browse(cr,uid,wh_id)
        return wh.lot_stock_id.id

    _defaults={
        'name': lambda obj, cr, uid, context: '/',
        "date": lambda *a: time.strftime("%Y-%m-%d"),
        "requester_id": lambda self,cr,uid,context: uid,
        "department_id": _get_department,
        "location_id": _get_location,
        "state": lambda *a: "draft",
        "required_date": lambda *a: time.strftime("%Y-%m-%d"),
    }

    def onchange_product(self,cr,uid,ids,product_id,supplier_id,state):
        if state=='approved':
            return {}
        if not product_id:
            return {"value":{"name":False,"uom_id":False,"qty":False,"supplier_id":False}}
        prod=self.pool.get("product.product").browse(cr,uid,product_id)
        uom_id=prod.uom_id.id
        qty=0.0
        supplier_id=prod.seller_ids and prod.seller_ids[0].name.id or False
        vals={
            "name": "[%s]:%s %s"%(prod.default_code,prod.name ,prod.variants or ""),
            "uom_id": uom_id,
            "qty": qty if qty not in (0.0,1.0) else False,
            "supplier_id": supplier_id,
        }
        return {"value": vals}

    def btn_submit(self,cr,uid,ids,context={}):
        for pr in self.browse(cr,uid,ids):
            if not pr.qty:
                raise osv.except_osv("Error","Can't request zero quantity")
        for pr in self.browse(cr,uid,ids):
            if not pr.name or pr.name=='/':
                pr.write({'name': self.pool.get('ir.sequence').get(cr, uid, 'purchase.request')})
            pr.write({"state":"wait_head"})
        return True

    def btn_approve_head(self,cr,uid,ids,context={}):
        for pr in self.browse(cr,uid,ids):
            pr.write({"state":"wait_gm"})
        return True

    def btn_disapprove_gm(self,cr,uid,ids,context={}):
        for pr in self.browse(cr,uid,ids):
            pr.write({"state":"disapproved"})
        return True

    def btn_approve_gm(self,cr,uid,ids,context={}):
        for pr in self.browse(cr,uid,ids):
            pr.write({"state":"approved"})
        return True

    def make_po(self,cr,uid,ids,context=None):
        user=self.pool.get("res.users").browse(cr,uid,uid)
        company=user.company_id
        pr_groups={}
        origins={}
        for pr in self.browse(cr,uid,ids):
            if pr.state!="approved":
                raise osv.except_osv("Error","Purchase request %s is not approved"%pr.name)
            if not pr.product_id:
                raise osv.except_osv("Error","Missing product in purchase request %s"%pr.name)
            if not pr.supplier_id:
                raise osv.except_osv("Error","Missing supplier in purchase request %s"%pr.name)
            k=(pr.supplier_id.id,pr.location_id.id,pr.product_id.id,pr.desc,pr.uom_id.id)
            pr_groups.setdefault(k,[]).append(pr.id)
            origins.setdefault(k[:2],set([])).add(pr.name)
        po_ids={}
        for (supp_id,loc_id,prod_id,desc,uom_id),pr_ids in pr_groups.items():
            qty=0.0
            for pr in self.browse(cr,uid,ids):
                qty+=pr.qty
            k=(supp_id,loc_id)
            po_id=po_ids.get(k)
            origin=",".join(sorted(origins[k]))
            if not po_id:
                vals={
                    "partner_id": supp_id,
                    "location_id": loc_id,
                    "origin": origin,
                }
                po_id=self.pool.get("purchase.order").create(cr,uid,vals,context={"partner_id":supp_id})
                po_ids[k]=po_id
            supp=self.pool.get("res.partner").browse(cr,uid,supp_id)
            pl=supp.property_product_pricelist_purchase
            price=pl.price_get(prod_id,qty,supp_id,{"uom": uom_id})[pl.id]
            vals={
                "order_id": po_id,
                "product_id": prod_id,
                "product_qty": qty,
                "product_uom": uom_id,
                "name": desc,
                "price_unit": price,
                "date_planned": time.strftime("%Y-%m-%d"),
            }
            self.pool.get("purchase.order.line").create(cr,uid,vals)
            self.write(cr,uid,pr_ids,{"po_id":po_id})
        return po_ids.values()

    def update_state(self,cr,uid,ids):
        for pr in self.browse(cr,uid,ids):
            if pr.po_id:
                if pr.po_id.shipped:
                    new_state="received"
                else:
                    new_state="ordered"
                if new_state!=pr.state:
                    pr.write({"state":new_state})
        return True

    def btn_cancel(self,cr,uid,ids,context={}):
        print "btn_cancel", ids
        for pr in self.browse(cr,uid,ids):
            if pr.po_id.state!="cancel":
                raise osv.except_osv(_("Error"),_("Purchase orders for this request have to be canceled first"))
            pr.write({"state":"canceled"})
        return True

    def write(self,cr,uid,ids,vals,context={}):
        res=super(purchase_request,self).write(cr,uid,ids,vals,context)
        if type(ids)!=type([]):
            ids=[ids]
        self.update_state(cr,uid,ids)
        return res
purchase_request()

class purchase_order(osv.osv):
    _inherit="purchase.order"
    _columns={
        "pr_ids": fields.one2many("purchase.request","po_id","Purchase Requests",readonly=True),
    }

    def write(self,cr,uid,ids,vals,context={}):
        res=super(purchase_order,self).write(cr,uid,ids,vals,context)
        if type(ids)!=type([]):
            ids=[ids]
        pr_ids=set([])
        for po in self.browse(cr,uid,ids):
            for pr in po.pr_ids:
                pr_ids.add(pr.id)
        pr_ids=list(pr_ids)
        self.pool.get("purchase.request").update_state(cr,uid,pr_ids)
        return res
purchase_order()

class mrp_procurement(osv.osv):
    _inherit="mrp.procurement"
    _columns={
        "pr_id": fields.many2one("purchase.request","Purchase Request"),
    }

    def make_pr(self, cr, uid, ids, context={}):
        purchase_id = False
        company = self.pool.get('res.users').browse(cr, uid, uid, context).company_id
        for procurement in self.browse(cr, uid, ids):
            res_id = procurement.move_id.id
            partner = procurement.product_id.seller_ids[0].name
            partner_id = partner.id
            address_id = self.pool.get('res.partner').address_get(cr, uid, [partner_id], ['delivery'])['delivery']
            pricelist_id = partner.property_product_pricelist_purchase.id

            uom_id = procurement.product_id.uom_po_id.id

            qty = self.pool.get('product.uom')._compute_qty(cr, uid, procurement.product_uom.id, procurement.product_qty, uom_id)
            if procurement.product_id.seller_ids[0].qty:
                qty=max(qty,procurement.product_id.seller_ids[0].qty)

            price = self.pool.get('product.pricelist').price_get(cr, uid, [pricelist_id], procurement.product_id.id, qty, False, {'uom': uom_id})[pricelist_id]

            #Passing partner_id to context for purchase order line integrity of Line name
            context.update({'lang':partner.lang, 'partner_id':partner_id})

            product=self.pool.get('product.product').browse(cr,uid,procurement.product_id.id,context=context)

            vals = {
                'origin': procurement.origin,
                'supplier_id': partner_id,
                'location_id': procurement.location_id.id,
                'desc': product.partner_ref,
                'qty': qty,
                'product_id': procurement.product_id.id,
                'uom_id': uom_id,
                'reserv_id': res_id,
                'notes':product.description_purchase,
            }
            pr_id=self.pool.get("purchase.request").create(cr,uid,vals)

            self.write(cr, uid, [procurement.id], {'state':'running', 'pr_id':pr_id})
mrp_procurement()
